import logging

from descope import DescopeClient
from django.contrib.auth import models as auth_models
from django.core.cache import cache
from django.utils.functional import cached_property

from .settings import IS_STAFF_ROLE, IS_SUPERUSER_ROLE, PROJECT_ID

logger = logging.getLogger(__name__)


class DescopeUser(auth_models.User):
    class Meta:
        proxy = True

    # User is always active since Descioe will never issue a token for an
    # inactive user
    is_active = True
    _descope = DescopeClient(PROJECT_ID)

    def sync(self, session, refresh):
        self.token = session
        self.session = session.get("jwt")
        self.refresh = refresh
        self.user = session.get("user")
        self.firstSeen = session.get("firstSeen")
        self.username = self._me.get("userId")
        self.email = self._me.get("email")
        self.is_staff = IS_STAFF_ROLE in self._roles
        self.is_superuser = IS_SUPERUSER_ROLE in self._roles
        self.save()

    def __str__(self):
        return f"DescopeUser {self.username}"

    @cached_property
    def _me(self):
        return cache.get_or_set(
            f"descope_me:{self.username}", lambda: self._descope.me(self.refresh)
        )

    @cached_property
    def _roles(self):
        return self.token.get("roles", [])

    def get_username(self):
        return self.username



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

WEB_COMPONENT_SRC = getattr(
    settings, "DESCOPE_WEB_COMPONENT_SRC", "https://unpkg.com/@descope/web-component"
)

PROJECT_ID = getattr(settings, "DESCOPE_PROJECT_ID", None)
if not PROJECT_ID:
    raise ImproperlyConfigured('"DESCOPE_PROJECT_ID" is required!')

# Role names to create in Descope that will map to User attributes
IS_STAFF_ROLE = getattr(settings, "DESCOPE_IS_STAFF_ROLE", "is_staff")
IS_SUPERUSER_ROLE = getattr(settings, "DESCOPE_IS_SUPERUSER_ROLE", "is_superuser")



import os

from descope import REFRESH_SESSION_COOKIE_NAME, SESSION_COOKIE_NAME
from django import template
from django.middleware.csrf import get_token as csrf_token
from django.urls import reverse
from django.utils.crypto import get_random_string
from django.utils.safestring import mark_safe

from ..settings import PROJECT_ID, WEB_COMPONENT_SRC

register = template.Library()
CONTEXT_KEY = "descope_wc_included"


@register.simple_tag(takes_context=True)
def descope_flow(context, flow_id, success_redirect):
    script = ""
    if not context.get(CONTEXT_KEY):
        script += f'<script src="{WEB_COMPONENT_SRC}"></script>'
        context[CONTEXT_KEY] = True
    id = "descope-" + get_random_string(length=4)
    store_jwt_url = reverse("django_descope:store_jwt")
    flow = f"""
    <descope-wc id="{id}" project-id="{PROJECT_ID}" flow-id="{flow_id}"
        base-url="{os.environ.get('DESCOPE_BASE_URI', '')}"></descope-wc>
    <script>
        const descopeWcEle = document.getElementById('{id}');
        descopeWcEle.addEventListener('success', async (e) => {{
            const formData = new FormData();
            formData.append('{SESSION_COOKIE_NAME}', e.detail.sessionJwt);
            formData.append('{REFRESH_SESSION_COOKIE_NAME}', e.detail.refreshJwt);
            formData.append('csrfmiddlewaretoken','{csrf_token(context.request)}')

            await fetch("{store_jwt_url}", {{
                method: "POST",
                body: formData,
            }})
            document.location.replace("{success_redirect}")
        }});
    </script>
    """
    return mark_safe(script + flow)



import logging

from django.contrib.auth import login
from django.http import HttpRequest, HttpResponse

from .authentication import DescopeAuthentication

logger = logging.getLogger(__name__)


class DescopeMiddleware:
    _auth = DescopeAuthentication()

    def __init__(self, get_response: HttpResponse = None):
        self.get_response = get_response

    def __call__(self, request: HttpRequest):
        user = self._auth.authenticate(request)
        if user:
            login(request, user)
        return self.get_response(request)



import logging

from descope import REFRESH_SESSION_COOKIE_NAME, SESSION_COOKIE_NAME, DescopeClient
from descope.exceptions import AuthException
from django.contrib.auth import logout
from django.contrib.auth.backends import BaseBackend
from django.http import HttpRequest

from .models import DescopeUser
from .settings import PROJECT_ID

logger = logging.getLogger(__name__)


class DescopeAuthentication(BaseBackend):
    _dclient = DescopeClient(PROJECT_ID)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def authenticate(self, request: HttpRequest):
        session = request.session.get(SESSION_COOKIE_NAME)
        refresh = request.session.get(REFRESH_SESSION_COOKIE_NAME)

        logger.debug("Validating (and refreshing) Descope session")
        logger.debug("session %s", session)
        logger.debug("refresh %s", refresh)
        try:
            validated_token = self._dclient.validate_and_refresh_session(
                session, refresh
            )
        except AuthException as e:
            """
            Ask forgiveness, not permission.
                - Grace Hopper

            This exception will be thrown on every unauthenticated request to
            ensure logging out an invalid user.
            """
            logger.debug(e)
            logout(request)
            return None

        logger.debug(validated_token)
        return self.get_user(request, validated_token, refresh)

    def get_user(self, request: HttpRequest, validated_token=None, refresh_token=None):
        if validated_token:
            username = validated_token.get("userId") or validated_token.get("sub")
            user, created = DescopeUser.objects.get_or_create(username=username)
            user.sync(validated_token, refresh_token)
            request.session[SESSION_COOKIE_NAME] = user.session
            return user
        return None



import logging

from descope import REFRESH_SESSION_COOKIE_NAME, SESSION_COOKIE_NAME, DescopeClient
from django.http import HttpRequest, HttpResponseBadRequest, JsonResponse
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.cache import never_cache

from . import settings

# User = get_user_model()
logger = logging.getLogger(__name__)

descope_client = DescopeClient(project_id=settings.PROJECT_ID)


@method_decorator([never_cache], name="dispatch")
class StoreJwt(View):
    def post(self, request: HttpRequest):
        session = request.POST.get(SESSION_COOKIE_NAME)
        refresh = request.COOKIES.get(REFRESH_SESSION_COOKIE_NAME)
        if not refresh:
            refresh = request.POST.get(REFRESH_SESSION_COOKIE_NAME)

        if session and refresh:
            request.session[SESSION_COOKIE_NAME] = session
            request.session[REFRESH_SESSION_COOKIE_NAME] = refresh
            return JsonResponse({"success": True})

        return HttpResponseBadRequest()
